package crm

import (
	"github.com/gin-gonic/gin"
	"github.com/go-playground/validator/v10"
	"hongze/fms_api/controller/resp"
	"hongze/fms_api/global"
	"hongze/fms_api/models/crm"
)

// CompanySellerController CRM销售
type CompanySellerController struct{}

// List
// @Title 销售列表
// @Description 销售列表
// @Param   SellerType  query  int  false  "销售类型: 0-所有; 1-FICC; 2-权益"
// @Success 200 {object} crm.CompanySellerList
// @router /crm/company_seller/list [get]
func (rg *CompanySellerController) List(c *gin.Context) {
	var req crm.CompanySellerListReq
	if e := c.BindQuery(&req); e != nil {
		err, ok := e.(validator.ValidationErrors)
		if !ok {
			resp.FailData("参数解析失败", "Err:"+e.Error(), c)
			return
		}
		resp.FailData("参数解析失败", err.Translate(global.Trans), c)
		return
	}

	roleCodeArr := make([]string, 0)
	if req.SellerType == crm.CompanyProductRai {
		roleCodeArr = []string{
			crm.ROLE_TYPE_CODE_RAI_ADMIN,
			crm.ROLE_TYPE_CODE_RAI_DEPARTMENT,
			crm.ROLE_TYPE_CODE_RAI_GROUP,
			crm.ROLE_TYPE_CODE_RAI_SELLER,
		}
	} else {
		roleCodeArr = []string{
			crm.ROLE_TYPE_CODE_FICC_ADMIN,
			crm.ROLE_TYPE_CODE_FICC_DEPARTMENT,
			crm.ROLE_TYPE_CODE_FICC_GROUP,
			crm.ROLE_TYPE_CODE_FICC_TEAM,
			crm.ROLE_TYPE_CODE_FICC_SELLER,
		}
	}
	cond := `b.role_type_code IN ?`
	pars := make([]interface{}, 0)
	pars = append(pars, roleCodeArr)
	list, e := crm.GetCompanySellerByRoleCodes(cond, pars)
	if e != nil {
		resp.FailData("获取失败", "获取销售列表失败, Err:"+e.Error(), c)
		return
	}
	resp.OkData("获取成功", list, c)
}
