package fms

import (
	"hongze/fms_api/global"
	"time"
)

// ContractServiceTemplate 合同服务模板表
type ContractServiceTemplate struct {
	ServiceTemplateId int       `gorm:"primaryKey;column:service_template_id" json:"service_template_id"`
	ProductId         string    `gorm:"column:product_id" json:"product_id" description:"产品id"`
	Pid               int       `gorm:"column:pid" json:"pid" description:"父级id"`
	Title             string    `gorm:"column:title" json:"title" description:"套餐名称"`
	Value             string    `gorm:"column:value" json:"value" description:"对应的数据"`
	SelectType        string    `gorm:"column:select_type" json:"select_type" description:"选择器类型，单选还是多选，radio；单选：checkbox"`
	SelectName        string    `gorm:"column:select_name" json:"select_name" description:"选择器名称"`
	TableValue        string    `gorm:"column:table_value" json:"table_value" description:"表格数据，用于word生成时的json数据"`
	Remark            string    `gorm:"column:remark" json:"remark" description:"表头备注"`
	ChartPermissionId int       `gorm:"column:chart_permission_id" json:"chart_permission_id" description:"权限id"`
	CreateTime        time.Time `gorm:"column:create_time" json:"create_time" description:"创建时间"`
	ModifyTime        time.Time `gorm:"column:modify_time" json:"modify_time" description:"修改时间"`
	IsDelete          int       `gorm:"column:is_delete" json:"-" description:"删除状态: 0-正常; 1-删除"`
}

func (st *ContractServiceTemplate) TableName() string {
	return "contract_service_template"
}

type ContractServiceTemplateItem struct {
	ServiceTemplateId int    `json:"service_template_id" description:"套餐模板ID"`
	ProductId         int    `json:"product_id" description:"产品ID"`
	Pid               int    `json:"pid" description:"父级ID"`
	Title             string `json:"title" description:"套餐名称"`
	Value             string `json:"value" description:"对应的数据"`
	SelectType        string `json:"select_type" description:"选择器类型：radio-单选; checkbox-多选"`
	SelectName        string `json:"select_name" description:"选择器名称"`
	ChartPermissionId int    `json:"chart_permission_id" description:"权限ID"`
}

type ContractServiceTemplateMapItems struct {
	ContractServiceTemplateItem
	Children []*ContractServiceTemplateMapItems `json:"children"`
	Detail   []*ContractServiceDetail           `json:"detail"`
}

type ContractServiceTemplateMapItems2 struct {
	ServiceTemplateId int                                `json:"service_template_id" description:"套餐模板ID"`
	ProductId         int                                `json:"product_id" description:"产品id"`
	Pid               int                                `json:"pid" description:"父级id"`
	Title             string                             `json:"title" description:"套餐名称"`
	Value             string                             `json:"value" description:"对应的数据"`
	SelectType        string                             `json:"select_type" description:"选择器类型，单选还是多选，radio；单选：checkbox"`
	SelectName        string                             `json:"select_name" description:"选择器名称"`
	ChartPermissionId int                                `json:"chart_permission_id" description:"权限id"`
	Children          []*ContractServiceTemplateMapItems `json:"children"`
	Detail            []*ContractServiceDetail           `json:"detail"`
}

// ContractServiceListReq CRM系统合同套餐-列表请求体
type ContractServiceListReq struct {
	ProductId int `json:"product_id" form:"product_id" description:"产品: 1-FICC; 2-权益"`
}

func GetContractServiceTemplateMapByProductId(productId int) (items []*ContractServiceTemplateItem, err error) {
	items = make([]*ContractServiceTemplateItem, 0)
	err = global.DEFAULT_MYSQL.Table("contract_service_template").
		Where("product_id = ? AND pid = 0 AND is_delete = 0", productId).
		Find(&items).Error
	return
}

func GetContractServiceTemplateMapByParentId(parentId int) (items []*ContractServiceTemplateItem, err error) {
	items = make([]*ContractServiceTemplateItem, 0)
	err = global.DEFAULT_MYSQL.Table("contract_service_template").
		Where("pid = ? AND is_delete = 0", parentId).
		Find(&items).Error
	return
}

// GetContractServiceTemplateById 主键获取合同套餐模板
func GetContractServiceTemplateById(templateId int) (item *ContractServiceTemplate, err error) {
	err = global.DEFAULT_MYSQL.Model(ContractServiceTemplate{}).
		Where("service_template_id = ? AND is_delete = 0", templateId).
		First(&item).Error
	return
}
